@@ -5,7 +5,8 @@ function( txt, x, y, wd, ht,
           col.border="black",
           col.bg="transparent" )
 {
-rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2, lwd=lwd, border=col.border, col=col.bg )
+rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2,
+      lwd=lwd, border=col.border, col=col.bg )
 text( x, y, txt, font=font, col=col.txt )
 invisible( c( x, y, wd, ht ) )
 }
@@ -38,6 +39,7 @@ arrows( x1 + (x2-x1)*(1-fr)/2,
         x2 - (x2-x1)*(1-fr)/2,
         y2 - (y2-y1)*(1-fr)/2, angle=a, lwd=lwd, ... )
 }
+
 std.vec <-
 function( a, b )
 {
@@ -88,6 +90,12 @@ invisible( list( x = bx1*(1-pos)+bx2*pos,
 
 boxes <- function (obj, ...) UseMethod("boxes")
 
+boxes.matrix <-
+function( obj, ... )
+  {
+  Epi:::boxes.Lexis( obj, ... )
+  }
+
 boxes.Lexis <-
 function( obj, file, detailed=FALSE,
                boxpos = FALSE,
@@ -121,23 +129,20 @@ function( obj, file, detailed=FALSE,
 {
 if( inherits(obj,"Lexis") )
   {
-  obj <- factorize( obj )
-  tm <- tmat( obj )
+  if( !is.factor(obj$lex.Cst) | !is.factor(obj$lex.Xst) ) obj <- factorize( obj )
+  tm <- tmat( obj, Y=TRUE )
   }
 else if( is.matrix(obj) & diff(dim(obj))==0 )
-       {
-       tm <- obj
-       # Put the transitions into D (only potentially used).
-       D <- tm
-       # Numbers in boxes ?
-       if( is.numeric(show.Y) )
-         {
-         Y <- show.Y
-         show.Y <- TRUE
-         }
-       }
+  {
+  tm <- obj
+  }
 else stop( "First argument must be a Lexis object or a square matrix.\n" )
 
+# Put the transitions into D and the diagnonal into Y.
+D <- tm
+diag( D ) <- NA
+Y <- diag( tm )
+
 # Derive state names, no. states and no. transitions
                       st.nam <- colnames( tm )
 if( is.null(st.nam) ) st.nam <- paste(1:ncol(tm))
@@ -156,6 +161,13 @@ if( inherits(obj,"Lexis") )
     }
   }
 
+# Explicitly given numbers in boxes ?
+if( is.numeric(show.Y) )
+  {
+  Y <- show.Y
+  show.Y <- TRUE
+  }
+
 # No extra line with person-years when they are NA
 if( show.Y ) pl.nam <- gsub( "\\\nNA",
                              "",
@@ -282,7 +294,8 @@ for( i in subset ) for( j in subset )
     }
   }
 # Redraw the boxes with white background to remove any arrows
-for( i in subset ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], col.bg="white" )
+for( i in subset ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i],
+                         lwd=lwd[i], col.bg="white" )
 # Then redraw the boxes again
 for( i in subset ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i],
                                 font=font[i],
@@ -344,4 +357,15 @@ boxes.Lexis( ', deparse( substitute( obj ) ),',
        offset.arr =', offset.arr, ')\t\t # offset of parallel arrows \n',
  file=file )
 }
+invisible( list( Boxes = data.frame( pl.nam = pl.nam,
+                                         xx = xx,
+                                         yy = yy,
+                                         wd = wd,
+                                         ht = ht,
+                                       font = font,
+                                        lwd = lwd,
+                                    col.txt = col.txt,
+                                 col.border = col.border,
+                                     col.bg = col.bg ),
+                  Tmat = tm ) )
 }
